define([
  "dojo/_base/declare",
  "dojo/dom-class",
  "dojo/dom-style",
  "dijit/form/ValidationTextBox"
], function(declare, domClass, domStyle, TextBox) {
  return declare("wagby.TextBox", [TextBox], {
    imeMode: "auto",          // auto|active|inactive|disabled "auto"
    cursorPosition: "left",   // left|right "left"
    postCreate: function() {
      this.inherited(arguments);
      this.setImeMode();
      this.setCursorPosition();
      this.setByteCheckMinMax();
      this.setErrorStyle();
    },

    setImeMode: function() {
      if (this.imeMode === "auto") {
        return;
      }
      domStyle.set(this.textbox, "imeMode", this.imeMode);
    },

    setCursorPosition: function() {
      if (this.cursorPosition === "left") {
        return;
      }
      domStyle.set(this.textbox, "textAlign", this.cursorPosition);
    },

    setByteCheckMinMax: function() {
      if (!this.byteSize || this.byteSize.indexOf(",") === -1) {
        return;
      }
      this.byteCheckMinMax = [];
      var byteSizeChecks = this.byteSize.split(",");
      for (var i=0,l=byteSizeChecks.length; i<l; i++) {
        this.byteCheckMinMax.push(parseInt(byteSizeChecks[i], 10));
      }
    },

    setErrorStyle: function() {
      if (!this.inputError || this.inputError === "false") {
        return;
      }
      domClass.add(this.domNode, "errorfield");
    },

    validator: function(/*anything*/ value, /*__Constraints*/ constraints) {
      if (this.type === "hidden") {
        return true;
      }
      var isValid = this.inherited(arguments);
      if (!this.byteSize || this._isEmpty(value)) {
        return isValid;
      }
      return this.isValidByteSize(value) && isValid;
    },

    isValidByteSize: function(value) {
      var byteCheckMin = this.byteCheckMinMax[0];
      var byteCheckMax = this.byteCheckMinMax[1];
      var valueByteSize = this.getUTF8ByteSize(value);
      if (byteCheckMin !== 0 && valueByteSize < byteCheckMin ||
          !isNaN(byteCheckMax) && valueByteSize > byteCheckMax) {
        return false;
      }
      return true;
    },

    getUTF8ByteSize: function(value) {
      try {
        return encodeURIComponent(value).replace(/%../g, "x").length;
      } catch(e) {
        console.error("encodeURIComponent error value: ", value);
      }
      return 0;
    },

  });
});
